import os
import subprocess


class OnlineRandomForest:
    def evaluate(self, xs_train, ys_train, xs_test, ys_test) -> float:
        self.write_eval_data(xs_train, ys_train, xs_test, ys_test)
        return self.execute()

    def execute(self) -> float:
        result = subprocess.run(
            [os.path.join(os.getcwd(), 'Online-Forest') + ' -c orf.conf --orf --t2'],
            stdout=subprocess.PIPE, shell=True, cwd=os.getcwd())

        return float(result.stdout.decode('utf-8').split('\n')[-3][52:-5])

    def write_eval_data(self, xs_train, ys_train, xs_test, ys_test):
        self.__write_dataset(xs_train, ys_train, 'train.libsvm', 10)
        self.__write_dataset(xs_test, ys_test, 'test.libsvm', 10)

    def __write_dataset(self, xs, ys, filename, classes_count):
        with open(filename, 'w') as f:
            f.write('{} {} {} {}\n'.format(len(xs), len(xs[0]), classes_count, 0))

            for (x, y_) in zip(xs, ys):
                self.__write_point(x, y_, f)

    def __write_point(self, x, y, file):
        file.write(str(y))

        for i, x_ in enumerate(x):
            if x_ != 0:
                file.write(' {}:{}'.format(i, str(x_)))

        file.write('\n')

    def configure(self, depth, tests, features, threshold, trees):
        with open('orf.conf', 'w') as f:
            f.write(
                '''\
Data:
{{
  trainData = "train.libsvm";
  testData = "test.libsvm";
}};
Tree:
{{
  maxDepth = {depth};
  numRandomTests = {tests};
  numProjectionFeatures = {features};
  counterThreshold = {threshold};
}};
Forest:
{{
  numTrees = {trees};
  numEpochs = 1;
  useSoftVoting = 1;
}};
Output:
{{
  savePath  = "/tmp/online-forest-";
  verbose = 1; // 0 = None
}};'''.format(depth=depth, tests=tests, features=features, threshold=threshold, trees=trees))
