from sklearn.datasets import load_digits

import numpy as np

from sklearn.model_selection import train_test_split

from OnlineRandomForest import OnlineRandomForest

digits = load_digits()
X, y = digits.data, digits.target

rng = np.random.RandomState(4)

test_sizes = [0.95, 0.9, 0.75, 0.5, 0.25, 0.1, 0.01]
rounds = 500


def main():
    orf = OnlineRandomForest()
    orf.configure(
        depth=20,
        tests=5,
        features=3,
        threshold=10,
        trees=100
    )

    test_errors = []

    for test_size in test_sizes:
        test_errors.append(score(orf, rounds, test_size))

    print(test_errors)
    # [0.34726816799999993, 0.18670210000000007, 0.08561868999999996, 0.05286095719999996, 0.04160888639999995, 0.037333329800000044, 0.03611112200000006]


def score(orf: OnlineRandomForest, rounds: int, test_size: float):
    accuracy_sum = 0

    print('Round 1 for test proportion {}.'.format(test_size))

    for r in range(rounds):
        if r % 10 == 9:
            print('Round {} for test proportion {}. Accuracy so far: {:.04f}'.format(r+1, test_size, accuracy_sum/r))

        X_train, X_test, y_train, y_test = \
            train_test_split(X, y, test_size=test_size, random_state=rng)

        accuracy = orf.evaluate(X_train, y_train, X_test, y_test)
        accuracy_sum += accuracy

    print('Round {} for test proportion {} finished with accuracy {:.04f}'.format(rounds, test_size, accuracy_sum/rounds))

    return accuracy_sum / rounds


if __name__ == '__main__':
    main()
